; Diashapes -- an diagram creation/manipulation program
; Copyright (C) 2007, 2008 Steffen Macke
;  
; diashapes.nsi : Nullsoft Installation System (NSIS) script
; Copyright (C) 2000-2004 Herman Bloggs, Steffen Macke
; Copyright (C) 2005-2008 Steffen Macke <sdteffen@sdteffen.de>
;  
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

; NOTE: this .NSI script is designed for NSIS v2.0

;--------------------------------
;Global Variables
Var name

;--------------------------------
;Configuration

;The name var is set in .onInit
Name $name

; Where the GTK+ binares are installed. Set in .onInit 
Var GTKBIN

; Uninstall only the installed files and folders
!define UninstLog "diashapes-uninstall.log"
Var UninstLog

; Add file macro
!macro File FilePath FileName
 !define FileID ${__LINE__}
 ;IfFileExists "$OUTDIR\${FileName}" NoExtract_${FileID}
 File "${FilePath}${FileName}"
 NoExtract_${FileID}:
 FileWrite $UninstLog "$OUTDIR\${FileName}$\r$\n"
 !undef FileID
!macroend
!define File "!insertmacro File"
 
; Set output path macro
!macro SetOutPath Path
 SetOutPath "${Path}"
 FileWrite $UninstLog "${PATH}$\r$\n"
!macroend
!define SetOutPath "!insertmacro SetOutPath"
 
Section -openlogfile
 SetOutPath "$INSTDIR"
 IfFileExists "$INSTDIR\${UninstLog}" +3
 FileOpen $UninstLog "$INSTDIR\${UninstLog}" w
 Goto +4
  SetFileAttributes "$INSTDIR\${UninstLog}" NORMAL
  FileOpen $UninstLog "$INSTDIR\${UninstLog}" a
  FileSeek $UninstLog 0 END
SectionEnd
; Allow to build two different versions of the installer
!ifdef DIASHAPES_INCLUDE_GTK
OutFile "diashapes-setup-${DIASHAPES_VERSION}.exe"
!else
OutFile "diashapes-setup-no-gtk-${DIASHAPES_VERSION}.exe"
!endif
SetCompressor LZMA
;DirShow show
ShowInstDetails show
ShowUninstDetails show
SetDateSave on

; $name and $INSTDIR are set in .onInit function..

!include "MUI.nsh"
;!include "Sections.nsh"

;--------------------------------
;Defines
!define DIA_VERSION_CHECK			"0.96.1"
!define DIA_REG_KEY					"SOFTWARE\Dia"
!define DIASHAPES_REG_KEY			"SOFTWARE\Diashapes"
!define DIA_UNINSTALL_KEY			"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\Diashapes"
!define HKLM_APP_PATHS_KEY			"SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\diashapes.exe"
!define DIA_STARTUP_RUN_KEY			"SOFTWARE\Microsoft\Windows\CurrentVersion\Run"
!define DIA_UNINST_EXE				"diashapes-${DIASHAPES_VERSION}-uninstall.exe"

XPStyle On
  
VIAddVersionKey "ProductName" "Diashapes for Windows"
VIAddVersionKey "CompanyName" "Steffen Macke"
VIAddVersionKey "LegalCopyright" "(c) 2007, 2008 Steffen Macke"
VIAddVersionKey "FileDescription" "Diashapes for Windows Installer"
VIAddVersionKey "FileVersion" "0.0.1.0"
VIProductVersion "0.0.1.0"

;--------------------------------
;Modern UI Configuration

  !define MUI_ICON				"dia-install.ico"
  !define MUI_UNICON				"dia-uninstall.ico"
  !define MUI_WELCOMEFINISHPAGE_BITMAP 		"dia-intro.bmp"
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP		"dia-header.bmp"
  !define MUI_HEADERIMAGE_UNBITMAP		"dia-header.bmp"

  ; Alter License section
  !define MUI_LICENSEPAGE_BUTTON		$(DIA_LICENSE_BUTTON)
  !define MUI_LICENSEPAGE_TEXT_BOTTOM		$(DIA_LICENSE_BOTTOM_TEXT)

  !define MUI_COMPONENTSPAGE_SMALLDESC
  !define MUI_ABORTWARNING

  ;Finish Page config
  !define MUI_FINISHPAGE_RUN			"$INSTDIR\bin\diashapes.exe"
  !define MUI_FINISHPAGE_RUN_NOTCHECKED
  !define MUI_FINISHPAGE_LINK			$(DIA_FINISH_VISIT_WEB_SITE)
  !define MUI_FINISHPAGE_LINK_LOCATION          "http://dia-installer.de/"

;--------------------------------
;Pages
  
  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE			"..\..\COPYING"
  !insertmacro MUI_PAGE_COMPONENTS

  ; Dia install dir page
  !insertmacro MUI_PAGE_DIRECTORY

  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  !insertmacro MUI_UNPAGE_WELCOME
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  !insertmacro MUI_UNPAGE_FINISH

;--------------------------------
;Languages
 
  ;; English goes first because its the default. The rest are
  ;; in alphabetical order (at least the strings actually displayed
  ;; will be).

  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Translations

  !define DIA_DEFAULT_LANGFILE "locale\english.nsh"

  !include "langmacros.nsh"
  
  !insertmacro DIA_MACRO_INCLUDE_LANGFILE "ENGLISH" "locale\english.nsh"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Start Install Sections ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
;--------------------------------
;Dia Install Section

Section $(DIA_SECTION_TITLE) SecDia
  SetOverwrite On
  SectionIn 1 RO

  ; Check install rights..
  Call CheckUserInstallRights
  Pop $R0

  StrCmp $R0 "NONE" dia_none
  StrCmp $R0 "HKLM" dia_hklm dia_hkcu
   
  dia_hklm:
    WriteRegStr HKLM "${HKLM_APP_PATHS_KEY}" "" "$INSTDIR\bin\diashapes.exe"
    WriteRegStr HKLM "${HKLM_APP_PATHS_KEY}" "Path" "$R1\bin;$GTKBIN"
    WriteRegStr HKLM "${HKLM_CMD_PATHS_KEY}" "" "$INSTDIR\bin\diashapes.exe"
    WriteRegStr HKLM "${HKLM_CMD_PATHS_KEY}" "Path" "$R1\bin;$GTKBIN"
    WriteRegStr HKLM ${DIASHAPES_REG_KEY} "" "$INSTDIR"
    WriteRegStr HKLM ${DIASHAPES_REG_KEY} "Version" "${DIASHAPES_VERSION}"
    WriteRegStr HKLM "${DIA_UNINSTALL_KEY}" "DisplayName" $(DIA_UNINSTALL_DESC)
    WriteRegStr HKLM "${DIA_UNINSTALL_KEY}" "UninstallString" "$INSTDIR\${DIA_UNINST_EXE}"
    WriteRegStr HKLM "${DIA_UNINSTALL_KEY}" "DisplayIcon" "$INSTDIR\${DIA_UNINST_EXE}"
    ; Sets scope of the desktop and Start Menu entries for all users.
    SetShellVarContext "all"
    Goto dia_install_files

  dia_hkcu:
    WriteRegStr HKCU ${DIASHAPES_REG_KEY} "" "$INSTDIR"
    WriteRegStr HKCU ${DIASHAPES_REG_KEY} "Version" "${DIASHAPES_VERSION}"
    WriteRegStr HKCU "${DIA_UNINSTALL_KEY}" "DisplayName" $(DIA_UNINSTALL_DESC)
    WriteRegStr HKCU "${DIA_UNINSTALL_KEY}" "UninstallString" "$INSTDIR\${DIA_UNINST_EXE}"
    WriteRegStr HKCU "${DIA_UNINSTALL_KEY}" "DisplayIcon" "$INSTDIR\${DIA_UNINST_EXE}"

    Goto dia_install_files

  dia_none:

  dia_install_files:

${SetOutPath} "$INSTDIR"

${SetOutPath} "$INSTDIR\bin"
; jEdit replacement: (^.*\\)(.*)$ 
; \${File} "$1" "$2"
${File} "..\..\devcpp\" "diashapes.exe"
${File} "..\..\..\bin\" "libgsf-1-114.dll"
${File} "..\..\..\bin\" "libcurl.dll"
${File} "..\..\..\bin\" "libeay32.dll"
${File} "..\..\..\bin\" "libssl32.dll"
${SetOutPath} "$INSTDIR\share"
${SetOutPath} "$INSTDIR\share\diashapes"
${File} "..\..\share\diashapes\" "diashapes.glade"

!ifdef DIASHAPES_INCLUDE_GTK
${SetOutPath} "$INSTDIR\bin"
;${File} "..\..\..\bin\" "charset.dll"
;${File} "..\..\..\bin\" "gspawn-win32-helper-console.exe"
;${File} "..\..\..\bin\" "gspawn-win32-helper.exe"
${File} "..\..\..\bin\" "iconv.dll"
;${File} "..\..\..\bin\" "iconv.exe"
${File} "..\..\..\bin\" "intl.dll"
${File} "..\..\..\bin\" "jpeg62.dll"
${File} "..\..\..\bin\" "libatk-1.0-0.dll"
${File} "..\..\..\bin\" "libcairo-2.dll"
;${File} "..\..\..\bin\" "libexpat.dll"
;${File} "..\..\..\bin\" "libfontconfig-1.dll"
;${File} "..\..\..\bin\" "freetype6.dll"
; TODO: Remove the following, once gladewin32 has switched to freetype6.dll
;${File} "..\..\..\bin\" "libfreetype-6.dll"
${File} "..\..\..\bin\" "libgdk-win32-2.0-0.dll"
${File} "..\..\..\bin\" "libgdk_pixbuf-2.0-0.dll"
${File} "..\..\..\bin\" "libglib-2.0-0.dll"
${File} "..\..\..\bin\" "libgmodule-2.0-0.dll"
${File} "..\..\..\bin\" "libgobject-2.0-0.dll"
${File} "..\..\..\bin\" "libgthread-2.0-0.dll"
${File} "..\..\..\bin\" "libgtk-win32-2.0-0.dll"
${File} "..\..\..\bin\" "libpango-1.0-0.dll"
${File} "..\..\..\bin\" "libpangocairo-1.0-0.dll"
${File} "..\..\..\bin\" "libpangoft2-1.0-0.dll"
${File} "..\..\..\bin\" "libpangowin32-1.0-0.dll"
;${File} "..\..\..\bin\" "libpng12.dll"
${File} "..\..\..\bin\" "libpng13.dll"
;${File} "..\..\..\bin\" "librle3.dll"
${File} "..\..\..\bin\" "libtiff3.dll"
${File} "..\..\..\bin\" "libxml2.dll"
${File} "..\..\..\bin\" "pango-querymodules.exe"
;${File} "..\..\..\bin\" "xmlparse.dll"
;${File} "..\..\..\bin\" "xmltok.dll"
${File} "..\..\..\bin\" "zlib1.dll"
${SetOutPath} "$INSTDIR\etc"
;${SetOutPath} "$INSTDIR\etc\fonts"
;${File} "..\..\..\etc\fonts\" "fonts.conf"
${SetOutPath} "$INSTDIR\etc\gtk-2.0"
${File} "..\..\..\etc\gtk-2.0\" "gdk-pixbuf.loaders"
${File} "..\..\..\etc\gtk-2.0\" "gtk.immodules"
# Workaround 0.96.1 installer bug
RMDir "$INSTDIR\etc\gtk-2.0\gtkrc"

${File} "..\..\..\share\themes\MS-Windows\gtk-2.0\" "gtkrc"
${SetOutPath} "$INSTDIR\etc\pango"
${File} "..\..\..\etc\pango\" "pango.aliases"
${File} "..\..\..\etc\pango\" "pango.modules"
${SetOutPath} "$INSTDIR\lib"
${SetOutPath} "$INSTDIR\lib\gtk-2.0"
${SetOutPath} "$INSTDIR\lib\gtk-2.0\2.10.0"
${SetOutPath} "$INSTDIR\lib\gtk-2.0\2.10.0\engines"
${File} "..\..\..\lib\gtk-2.0\2.10.0\engines\" "libpixmap.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\engines\" "libwimp.dll"
${SetOutPath} "$INSTDIR\lib\gtk-2.0\2.10.0\immodules"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-am-et.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-cedilla.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-cyrillic-translit.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-ime.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-inuktitut.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-ipa.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-ti-er.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-ti-et.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\immodules\" "im-viqr.dll"
${SetOutPath} "$INSTDIR\lib\gtk-2.0\2.10.0\loaders"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-ani.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-bmp.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-gif.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-ico.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-jpeg.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-pcx.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-png.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-pnm.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-ras.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-tga.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-tiff.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-wbmp.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-xbm.dll"
${File} "..\..\..\lib\gtk-2.0\2.10.0\loaders\" "libpixbufloader-xpm.dll"

${SetOutPath} "$INSTDIR\share"
${SetOutPath} "$INSTDIR\share\themes"
${SetOutPath} "$INSTDIR\share\themes\Default"
${SetOutPath} "$INSTDIR\share\themes\Default\gtk-2.0-key"
${File} "..\..\..\share\themes\Default\gtk-2.0-key\" "gtkrc"
${SetOutPath} "$INSTDIR\share\themes\Emacs"
${SetOutPath} "$INSTDIR\share\themes\Emacs\gtk-2.0-key"
${File} "..\..\..\share\themes\Emacs\gtk-2.0-key\" "gtkrc"
${SetOutPath} "$INSTDIR\share\themes\MS-Windows"
${SetOutPath} "$INSTDIR\share\themes\MS-Windows\gtk-2.0"
${File} "..\..\..\share\themes\MS-Windows\gtk-2.0\" "gtkrc"
${SetOutPath} "$INSTDIR\share\themes\Raleigh"
${SetOutPath} "$INSTDIR\share\themes\Raleigh\gtk-2.0"
${File} "..\..\..\share\themes\Raleigh\gtk-2.0\" "gtkrc"

!endif    
    SetShellVarContext all
    
    CreateDirectory "$SMPROGRAMS\Diashapes"
    CreateShortCut "$SMPROGRAMS\Diashapes\Diashapes.lnk" "$INSTDIR\bin\diashapes.exe"
    SetOutPath "$INSTDIR"
	
    ; write out uninstaller
    SetOverwrite on
    WriteUninstaller "$INSTDIR\${DIA_UNINST_EXE}"
    SetOverwrite off
    CreateShortCut "$SMPROGRAMS\Diashapes\Uninstall.lnk" "$INSTDIR\${DIA_UNINST_EXE}"
    
  done:
SectionEnd ; end of default Dia section
  
Section -closelogfile
 FileClose $UninstLog
 SetFileAttributes "$INSTDIR\${UninstLog}" READONLY|SYSTEM|HIDDEN
SectionEnd

;--------------------------------
;Uninstaller Section

Section Uninstall
  
  Call un.CheckUserInstallRights
  Pop $R0
  StrCmp $R0 "NONE" no_rights
  StrCmp $R0 "HKCU" try_hkcu try_hklm

  try_hkcu:
    ReadRegStr $R0 HKCU ${DIASHAPES_REG_KEY} ""
    StrCmp $R0 $INSTDIR 0 cant_uninstall
      ; HKCU install path matches our INSTDIR.. so uninstall
      DeleteRegKey HKCU ${DIASHAPES_REG_KEY}
      DeleteRegKey HKCU "${DIA_UNINSTALL_KEY}"
      Goto cont_uninstall

  try_hklm:
    ReadRegStr $R0 HKLM ${DIASHAPES_REG_KEY} ""
    StrCmp $R0 $INSTDIR 0 try_hkcu
      ; HKLM install path matches our INSTDIR.. so uninstall
      DeleteRegKey HKLM ${DIASHAPES_REG_KEY}
      DeleteRegKey HKLM "${DIA_UNINSTALL_KEY}"
      DeleteRegKey HKLM "${HKLM_APP_PATHS_KEY}"
      DeleteRegKey HKLM "${HKLM_CMD_PATHS_KEY}"
      ; Sets start menu and desktop scope to all users..
      SetShellVarContext "all"

  cont_uninstall:
    
    ; The WinPrefs plugin may have left this behind..
    DeleteRegValue HKCU "${DIA_STARTUP_RUN_KEY}" "Diashapes"
    DeleteRegValue HKLM "${DIA_STARTUP_RUN_KEY}" "Diashapes"
    ; Remove Language preference info
    DeleteRegKey HKCU ${DIASHAPES_REG_KEY} ;${MUI_LANGDLL_REGISTRY_ROOT} ${MUI_LANGDLL_REGISTRY_KEY}
    
    ; Can't uninstall if uninstall.log is missing!
 IfFileExists "$INSTDIR\${UninstLog}" +3
  MessageBox MB_OK|MB_ICONSTOP "${UninstLog} not found!"
  Goto no_uninstlog
 
 Push $R0
 SetFileAttributes "$INSTDIR\${UninstLog}" NORMAL
 FileOpen $UninstLog "$INSTDIR\${UninstLog}" r
 
 LoopRead:
  ClearErrors
   FileRead $UninstLog $R0
   IfErrors LoopDone
 
   Push $R0
    Call un.TrimNewLines
   Pop $R0
   IfFileExists "$R0\*.*" 0 +3
    RMDir $R0  #is dir
   Goto LoopRead
    Delete $R0 #is file
 
    Goto LoopRead
   LoopDone:
   FileClose $UninstLog
   Delete "$INSTDIR\${UninstLog}"
   Pop $R0
    Delete "$INSTDIR\${DIA_UNINST_EXE}"
    
    no_uninstlog:
    ; Shortcuts..
    Delete "$SMPROGRAMS\Diashapes\Diashapes.lnk"
    Delete "$SMPROGRAMS\Diashapes\Uninstall.lnk"
    RMDir "$SMPROGRAMS\Diashapes"
    
    Goto done

  cant_uninstall:
    IfSilent skip_mb
    MessageBox MB_OK $(un.DIA_UNINSTALL_ERROR_1) IDOK
    skip_mb:
    Quit

  no_rights:
    IfSilent skip_mb1
    MessageBox MB_OK $(un.DIA_UNINSTALL_ERROR_2) IDOK
    skip_mb1:
    Quit

  done:
SectionEnd ; end of uninstall section

;--------------------------------
;Descriptions
!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
!insertmacro MUI_DESCRIPTION_TEXT ${SecDia} \
	$(DIA_SECTION_DESCRIPTION)

!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Functions

Function CheckUserInstallRights
	ClearErrors
	UserInfo::GetName
	IfErrors Win9x
	Pop $0
	UserInfo::GetAccountType
	Pop $1

	StrCmp $1 "Admin" 0 +3
                StrCpy $1 "HKLM"
		Goto done
	StrCmp $1 "Power" 0 +3
                StrCpy $1 "HKLM"
		Goto done
	StrCmp $1 "User" 0 +3
		StrCpy $1 "HKCU"
		Goto done
	StrCmp $1 "Guest" 0 +3
		StrCpy $1 "NONE"
		Goto done
	; Unknown error
	StrCpy $1 "NONE"
        Goto done

	Win9x:
		StrCpy $1 "HKLM"

	done:
        Push $1
FunctionEnd

Function un.CheckUserInstallRights
	ClearErrors
	UserInfo::GetName
	IfErrors Win9x
	Pop $0
	UserInfo::GetAccountType
	Pop $1

	StrCmp $1 "Admin" 0 +3
                StrCpy $1 "HKLM"
		Goto done
	StrCmp $1 "Power" 0 +3
                StrCpy $1 "HKLM"
		Goto done
	StrCmp $1 "User" 0 +3
		StrCpy $1 "HKCU"
		Goto done
	StrCmp $1 "Guest" 0 +3
		StrCpy $1 "NONE"
		Goto done
	; Unknown error
	StrCpy $1 "NONE"
        Goto done

	Win9x:
		StrCpy $1 "HKLM"

	done:
        Push $1
FunctionEnd

;
; Usage:
;   Push $0 ; Path string
;   Call VerifyDir
;   Pop $0 ; 0 - Bad path  1 - Good path
;
Function VerifyDir
  Pop $0
  Loop:
    IfFileExists $0 dir_exists
    StrCpy $1 $0 ; save last
    Push $0
    Call GetParent
    Pop $0
    StrLen $2 $0
    ; IfFileExists "C:" on xp returns true and on win2k returns false
    ; So we're done in such a case..
    StrCmp $2 "2" loop_done
    Goto Loop

  loop_done:
    StrCpy $1 "$0\diaFooB"
    ; Check if we can create dir on this drive..
    ClearErrors
    CreateDirectory $1
    IfErrors DirBad DirGood

  dir_exists:
    ClearErrors
    FileOpen $1 "$0\diafoo.bar" w
    IfErrors PathBad PathGood

    DirGood:
      RMDir $1
      Goto PathGood1

    DirBad:
      RMDir $1
      Goto PathBad1

    PathBad:
      FileClose $1
      Delete "$0\diafoo.bar"
      PathBad1:
      StrCpy $0 "0"
      Push $0
      Return

    PathGood:
      FileClose $1
      Delete "$0\diafoo.bar"
      PathGood1:
      StrCpy $0 "1"
      Push $0
FunctionEnd

Function .onVerifyInstDir
  Push $INSTDIR
  Call VerifyDir
  Pop $0
  StrCmp $0 "0" 0 dir_good
    Abort
  dir_good:
FunctionEnd

; GetParent
; input, top of stack  (e.g. C:\Program Files\Poop)
; output, top of stack (replaces, with e.g. C:\Program Files)
; modifies no other variables.
;
; Usage:
;   Push "C:\Program Files\Directory\Whatever"
;   Call GetParent
;   Pop $R0
;   ; at this point $R0 will equal "C:\Program Files\Directory"
Function GetParent
   Exch $0 ; old $0 is on top of stack
   Push $1
   Push $2
   StrCpy $1 -1
   loop:
     StrCpy $2 $0 1 $1
     StrCmp $2 "" exit
     StrCmp $2 "\" exit
     IntOp $1 $1 - 1
   Goto loop
   exit:
     StrCpy $0 $0 $1
     Pop $2
     Pop $1
     Exch $0 ; put $0 on top of stack, restore $0 to original value
FunctionEnd

!define SECTION_ENABLE   0xFFFFFFEF
Function .onInit
  StrCpy $name "Diashapes ${DIASHAPES_VERSION}"
  !define MUI_LANGDLL_ALLLANGUAGES
  ;!insertmacro MUI_LANGDLL_DISPLAY
  ClearErrors
  
  ; If install path was set on the command, use it.
  StrCmp $INSTDIR "" 0 instdir_done
  
  ReadRegStr $0 HKLM ${DIA_REG_KEY} "Version"
  StrCpy $1 $0 6
  StrCmp $1 "0.96.1" HasDia HasNoDia
  HasNoDia:
  MessageBox MB_OK|MB_ICONSTOP "Please install Dia 0.96.1 first."
  Abort "Please install Dia 0.96.1 first"
  
  HasDia:
  ReadRegStr $INSTDIR HKLM ${DIA_REG_KEY} ""
  
  Call CheckUserInstallRights
  Pop $0

  StrCmp $0 "HKLM" 0 user_dir
    StrCpy $INSTDIR "$PROGRAMFILES\Dia"
    Goto instdir_done
  user_dir:
    StrCpy $2 "$SMPROGRAMS"
    Push $2
    Call GetParent
    Call GetParent
    Pop $2
    StrCpy $INSTDIR "$2\Dia"

  instdir_done:
  StrCpy $GTKBIN "$INSTDIR\bin"
  
  !ifdef DIASHAPES_INCLUDE_GTK
  StrCpy $GTKBIN "$INSTDIR\bin"
  !endif
FunctionEnd

Function un.TrimNewlines
   Exch $R0
   Push $R1
   Push $R2
   StrCpy $R1 0
 
 loop:
   IntOp $R1 $R1 - 1
   StrCpy $R2 $R0 1 $R1
   StrCmp $R2 "$\r" loop
   StrCmp $R2 "$\n" loop
   IntOp $R1 $R1 + 1
   IntCmp $R1 0 no_trim_needed
   StrCpy $R0 $R0 $R1
 
 no_trim_needed:
   Pop $R2
   Pop $R1
   Exch $R0
 FunctionEnd

; GetParameterValue
; Chris Morgan<cmorgan@alum.wpi.edu> 5/10/2004
; -Updated 4/7/2005 to add support for retrieving a command line switch
;  and additional documentation
;
; Searches the command line input, retrieved using GetParameters, for the
; value of an option given the option name.  If no option is found the
; default value is placed on the top of the stack upon function return.
;
; This function can also be used to detect the existence of just a
; command line switch like /OUTPUT  Pass the default and "OUTPUT"
; on the stack like normal.  An empty return string "" will indicate
; that the switch was found, the default value indicates that
; neither a parameter or switch was found.
;
; Inputs - Top of stack is default if parameter isn't found,
;  second in stack is parameter to search for, ex. "OUTPUT"
; Outputs - Top of the stack contains the value of this parameter
;  So if the command line contained /OUTPUT=somedirectory, "somedirectory"
;  will be on the top of the stack when this function returns
;
; Register usage
;$R0 - default return value if the parameter isn't found
;$R1 - input parameter, for example OUTPUT from the above example
;$R2 - the length of the search, this is the search parameter+2
;      as we have '/OUTPUT='
;$R3 - the command line string
;$R4 - result from StrStr calls
;$R5 - search for ' ' or '"'
 
Function GetParameterValue
  Exch $R0  ; get the top of the stack(default parameter) into R0
  Exch      ; exchange the top of the stack(default) with
            ; the second in the stack(parameter to search for)
  Exch $R1  ; get the top of the stack(search parameter) into $R1
 
  ;Preserve on the stack the registers used in this function
  Push $R2
  Push $R3
  Push $R4
  Push $R5
 
  Strlen $R2 $R1+2    ; store the length of the search string into R2
 
  Call GetParameters  ; get the command line parameters
  Pop $R3             ; store the command line string in R3
 
  # search for quoted search string
  StrCpy $R5 '"'      ; later on we want to search for a open quote
  Push $R3            ; push the 'search in' string onto the stack
  Push '"/$R1='       ; push the 'search for'
  Call StrStr         ; search for the quoted parameter value
  Pop $R4
  StrCpy $R4 $R4 "" 1   ; skip over open quote character, "" means no maxlen
  StrCmp $R4 "" "" next ; if we didn't find an empty string go to next
 
  # search for non-quoted search string
  StrCpy $R5 ' '      ; later on we want to search for a space since we
                      ; didn't start with an open quote '"' we shouldn't
                      ; look for a close quote '"'
  Push $R3            ; push the command line back on the stack for searching
  Push '/$R1='        ; search for the non-quoted search string
  Call StrStr
  Pop $R4
 
  ; $R4 now contains the parameter string starting at the search string,
  ; if it was found
next:
  StrCmp $R4 "" check_for_switch ; if we didn't find anything then look for
                                 ; usage as a command line switch
  # copy the value after /$R1= by using StrCpy with an offset of $R2,
  # the length of '/OUTPUT='
  StrCpy $R0 $R4 "" $R2  ; copy commandline text beyond parameter into $R0
  # search for the next parameter so we can trim this extra text off
  Push $R0
  Push $R5            ; search for either the first space ' ', or the first
                      ; quote '"'
                      ; if we found '"/output' then we want to find the
                      ; ending ", as in '"/output=somevalue"'
                      ; if we found '/output' then we want to find the first
                      ; space after '/output=somevalue'
  Call StrStr         ; search for the next parameter
  Pop $R4
  StrCmp $R4 "" done  ; if 'somevalue' is missing, we are done
  StrLen $R4 $R4      ; get the length of 'somevalue' so we can copy this
                      ; text into our output buffer
  StrCpy $R0 $R0 -$R4 ; using the length of the string beyond the value,
                      ; copy only the value into $R0
  goto done           ; if we are in the parameter retrieval path skip over
                      ; the check for a command line switch
 
; See if the parameter was specified as a command line switch, like '/output'
check_for_switch:
  Push $R3            ; push the command line back on the stack for searching
  Push '/$R1'         ; search for the non-quoted search string
  Call StrStr
  Pop $R4
  StrCmp $R4 "" done  ; if we didn't find anything then use the default
  StrCpy $R0 ""       ; otherwise copy in an empty string since we found the
                      ; parameter, just didn't find a value
 
done:
  Pop $R5
  Pop $R4
  Pop $R3
  Pop $R2
  Pop $R1
  Exch $R0 ; put the value in $R0 at the top of the stack
FunctionEnd

 ; GetParameters
 ; input, none
 ; output, top of stack (replaces, with e.g. whatever)
 ; modifies no other variables.
 
Function GetParameters
 
  Push $R0
  Push $R1
  Push $R2
  Push $R3
  
  StrCpy $R2 1
  StrLen $R3 $CMDLINE
  
  ;Check for quote or space
  StrCpy $R0 $CMDLINE $R2
  StrCmp $R0 '"' 0 +3
    StrCpy $R1 '"'
    Goto loop
  StrCpy $R1 " "
  
  loop:
    IntOp $R2 $R2 + 1
    StrCpy $R0 $CMDLINE 1 $R2
    StrCmp $R0 $R1 get
    StrCmp $R2 $R3 get
    Goto loop
  
  get:
    IntOp $R2 $R2 + 1
    StrCpy $R0 $CMDLINE 1 $R2
    StrCmp $R0 " " get
    StrCpy $R0 $CMDLINE "" $R2
  
  Pop $R3
  Pop $R2
  Pop $R1
  Exch $R0
 
FunctionEnd

!define StrStr "!insertmacro StrStr"
 
!macro StrStr ResultVar String SubString
  Push `${String}`
  Push `${SubString}`
  Call StrStr
  Pop `${ResultVar}`
!macroend
 
Function StrStr
/*After this point:
  ------------------------------------------
  $R0 = SubString (input)
  $R1 = String (input)
  $R2 = SubStringLen (temp)
  $R3 = StrLen (temp)
  $R4 = StartCharPos (temp)
  $R5 = TempStr (temp)*/
 
  ;Get input from user
  Exch $R0
  Exch
  Exch $R1
  Push $R2
  Push $R3
  Push $R4
  Push $R5
 
  ;Get "String" and "SubString" length
  StrLen $R2 $R0
  StrLen $R3 $R1
  ;Start "StartCharPos" counter
  StrCpy $R4 0
 
  ;Loop until "SubString" is found or "String" reaches its end
  loop:
    ;Remove everything before and after the searched part ("TempStr")
    StrCpy $R5 $R1 $R2 $R4
 
    ;Compare "TempStr" with "SubString"
    StrCmp $R5 $R0 done
    ;If not "SubString", this could be "String"'s end
    IntCmp $R4 $R3 done 0 done
    ;If not, continue the loop
    IntOp $R4 $R4 + 1
    Goto loop
  done:
 
/*After this point:
  ------------------------------------------
  $R0 = ResultVar (output)*/
 
  ;Remove part before "SubString" on "String" (if there has one)
  StrCpy $R0 $R1 `` $R4
 
  ;Return output to user
  Pop $R5
  Pop $R4
  Pop $R3
  Pop $R2
  Pop $R1
  Exch $R0
FunctionEnd
